﻿using BcGov.Fams3.Redis;
using BcGov.Fams3.Redis.Model;
using BcGov.Fams3.SearchApi.Contracts.Person;
using BcGov.Fams3.SearchApi.Contracts.PersonSearch;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;
using Moq;
using NUnit.Framework;
using SearchApi.Core.Test.Fake;
using SearchApi.Web.Configuration;
using SearchApi.Web.DeepSearch;
using SearchApi.Web.DeepSearch.Schema;
using SearchApi.Web.Notifications;
using SearchApi.Web.Test.Utils;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;

namespace SearchApi.Web.Test.DeepSearch
{
    public class DeepSearchServiceTest
    {

        private DeepSearchService _sut;
        private Mock<ILogger<DeepSearchService>> _loggerMock;
        private Mock<ISearchApiNotifier<PersonSearchAdapterEvent>> _searchApiNotifierMock;
        private Mock<IDeepSearchDispatcher> _dispatcherMock;
        private Mock<ICacheService> _cacheServiceMock;
        private Mock<IOptions<DeepSearchOptions>> _deepSearchOptionsMock;
        WaveSearchData wave;
        string dataPartner = "ICBC";
        string SearchRequestKey = "FirstTimeWave";
        string _Update_Parameter_SK = "000000_111111";
        string _Update_Parameter_SK_CCN = "000000_222222";
        PersonSearchCompleted completedStatus;
        PersonSearchCompleted completedStatusCCN;
        List<PersonFound> personFound;
        [SetUp]
        public void SetUp()
        {
            var waveitem1 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"ICBC\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"location\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem2 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"BCHYDRO\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"PHN\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem3 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"MSDPR\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"Employment\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem4 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"ICBC\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"location\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"newParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"location\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"payor\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"Payor\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem5 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"BCHYDRO\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"PHN\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"newParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"phn\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"payor\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"payor\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem6 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"MHSD\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"Employment\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"newParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"employment\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"payor\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"payor\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";


            var waveitem7 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"ICBC\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"location\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"93943589892\",\"type\":\"WorkSafeBCCCN\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem8 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"BCHYDRO\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"PHN\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"93943589892\",\"type\":\"WorkSafeBCCCN\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem9 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"MSDPR\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"Employment\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"93943589892\",\"type\":\"WorkSafeBCCCN\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
            var waveitem10 = "{\"searchRequestKey\":\"searchkey1\",\"currentWave\":2,\"dataPartner\":\"WORKSAFEBC\",\"allParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"location\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"PAYOR\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"93943589892\",\"type\":\"WorkSafeBCCCN\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"newParameter\":[{\"agency\":{\"notes\":\"string\",\"requestPriority\":\"Urgent\",\"code\":\"string\",\"requestId\":\"string\",\"reasonCode\":\"EnfPayAgr\",\"requestDate\":\"2020-09-10T18:53:01.884Z\",\"informationRequested\":[\"location\"],\"agent\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"payor\",\"owner\":\"Applicant\"},\"agentContact\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"email\":\"string\",\"locationCode\":\"string\",\"locationAddress\":\"string\"},\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.884Z\",\"dateOfDeath\":\"2020-09-10T18:53:01.884Z\",\"gender\":\"string\",\"dateDeathConfirmed\":true,\"incacerated\":\"string\",\"height\":\"string\",\"weight\":\"string\",\"hairColour\":\"string\",\"eyeColour\":\"string\",\"complexion\":\"string\",\"distinguishingFeatures\":\"string\",\"wearGlasses\":\"string\",\"type\":\"Payor\",\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"addresses\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"}],\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"names\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.884Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"relatedPersons\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"gender\":\"string\",\"dateOfBirth\":\"2020-09-10T18:53:01.885Z\",\"type\":\"string\"}],\"employments\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"employmentConfirmed\":true,\"incomeAssistance\":true,\"incomeAssistanceStatus\":\"string\",\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"},\"occupation\":\"string\",\"website\":\"string\"}],\"bankInfos\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"}],\"vehicles\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"ownershipType\":\"string\",\"vin\":\"string\",\"plateNumber\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"otherAssets\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"typeDescription\":\"string\",\"referenceDescription\":\"string\",\"referenceValue\":\"string\",\"owners\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}]}],\"compensationClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"claimantNumber\":\"string\",\"bankInfo\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"bankName\":\"string\",\"branch\":\"string\",\"transitNumber\":\"string\",\"accountNumber\":\"string\"},\"employer\":{\"name\":\"string\",\"ownerName\":\"string\",\"phones\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}],\"address\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactPerson\":\"string\"}}],\"insuranceClaims\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"claimType\":\"string\",\"claimStatus\":\"string\",\"claimNumber\":\"string\",\"adjustor\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"adjustorPhone\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"identifiers\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"value\":\"string\",\"type\":\"BCDriverLicense\",\"typeCode\":\"string\",\"issuedBy\":\"string\",\"owner\":\"Applicant\"}],\"insuredParties\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"name\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"firstName\":\"string\",\"lastName\":\"string\",\"middleName\":\"string\",\"otherName\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"},\"type\":\"string\",\"typeDescription\":\"string\",\"organization\":\"string\"}],\"claimCentre\":{\"location\":\"string\",\"contactAddress\":{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"type\":\"string\",\"addressLine1\":\"string\",\"addressLine2\":\"string\",\"addressLine3\":\"string\",\"stateProvince\":\"string\",\"city\":\"string\",\"countryRegion\":\"string\",\"zipPostalCode\":\"string\",\"owner\":\"Applicant\"},\"contactNumber\":[{\"referenceDates\":[{\"index\":0,\"key\":\"string\",\"value\":\"2020-09-10T18:53:01.885Z\"}],\"description\":\"string\",\"notes\":\"string\",\"phoneNumber\":\"string\",\"extension\":\"string\",\"type\":\"string\",\"owner\":\"Applicant\"}]}}],\"notes\":\"string\",\"searchRequestKey\":\"string\"}],\"numberOfRetries\":2,\"timeBetweenRetries\":40}";
           
            _loggerMock = LoggerUtils.LoggerMock<DeepSearchService>();
            _deepSearchOptionsMock = new Mock<IOptions<DeepSearchOptions>>();
            _cacheServiceMock = new Mock<ICacheService>();
            _searchApiNotifierMock = new Mock<ISearchApiNotifier<PersonSearchAdapterEvent>>();
            _dispatcherMock = new Mock<IDeepSearchDispatcher>();
            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{SearchRequestKey}-{dataPartner}"))
                .Returns(Task.FromResult(""));
            personFound = new List<PersonFound>();
            personFound.Add(new PersonFound { Identifiers = new List<PersonalIdentifier>() { new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value = "1230987" }, new PersonalIdentifier { Type = PersonalIdentifierType.SocialInsuranceNumber, Value = "123098712" } } });
            personFound.Add(new PersonFound { Identifiers = new List<PersonalIdentifier>() { new PersonalIdentifier { Type = PersonalIdentifierType.PersonalHealthNumber, Value = "4412234" }, new PersonalIdentifier { Type = PersonalIdentifierType.SocialInsuranceNumber, Value = "980765123" } } });
            _deepSearchOptionsMock.Setup(x => x.Value).Returns(new DeepSearchOptions
            {
                MaxWaveCount = 5
            });
            wave = new WaveSearchData
            {
                AllParameter = new List<Person>(),
                CurrentWave = 2,
                DataPartner = dataPartner,
                NewParameter = new List<Person>(),
                SearchRequestKey = SearchRequestKey,
                NumberOfRetries = 1,
                TimeBetweenRetries = 3
            };
            completedStatus = new FakePersonSearchCompleted
            {

                SearchRequestKey = _Update_Parameter_SK,
                SearchRequestId = Guid.NewGuid(),
                ProviderProfile = new FakeProviderProfile { Name = "ICBC", SearchSpeedType = SearchSpeedType.Fast },
                MatchedPersons = new List<PersonFound> {
                    new PersonFound { Identifiers= new List<PersonalIdentifier> {

                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="string" } ,
                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="1234567" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="---" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.SocialInsuranceNumber, Value="987654321" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.PersonalHealthNumber , Value="23456709182" }
                    }

                },
                    new PersonFound { Identifiers= new List<PersonalIdentifier> {

                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="string" } ,
                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="1234567" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.SocialInsuranceNumber, Value="0912452331" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.SocialInsuranceNumber, Value="--" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.PersonalHealthNumber , Value="0912882222" }
                    }
                    }
                }
            };

            completedStatusCCN = new FakePersonSearchCompleted
            {

                SearchRequestKey = _Update_Parameter_SK,
                SearchRequestId = Guid.NewGuid(),
                ProviderProfile = new FakeProviderProfile { Name = "WORKSAFEBC", SearchSpeedType = SearchSpeedType.Fast },
                MatchedPersons = new List<PersonFound> {
                    new PersonFound { Identifiers= new List<PersonalIdentifier> {

                      
                        new PersonalIdentifier { Type = PersonalIdentifierType.WorkSafeBCCCN, Value="93943589892" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.SocialInsuranceNumber, Value="726446123" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.PersonalHealthNumber , Value="9034856468" }
                    }

                }
                }
            };

            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-ICBC"))
    .Returns(Task.FromResult(waveitem7));

            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-BCHYDRO"))
              .Returns(Task.FromResult(waveitem8));

            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-MSDPR"))
             .Returns(Task.FromResult(waveitem9));
            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-WORKSAFEBC"))
        .Returns(Task.FromResult(waveitem10));

            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK_CCN}-ICBC", It.IsAny<object>()))
               .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK_CCN}-BCHYDRO", It.IsAny<object>()))
              .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK_CCN}-MSDPR", It.IsAny<object>()))
             .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK_CCN}-WORKSAFEBC", It.IsAny<object>()))
         .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.SearchKeys($"deepsearch-{_Update_Parameter_SK_CCN}*"))
           .Returns(Task.FromResult(new List<string>() { $"deepsearch-{_Update_Parameter_SK_CCN}-ICBC", $"deepsearch-{_Update_Parameter_SK_CCN}-BCHYDRO", $"deepsearch-{_Update_Parameter_SK_CCN}-MSDPR", $"deepsearch-{_Update_Parameter_SK_CCN}-WORKSAFEBC" }.AsEnumerable()));


            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK}-ICBC"))
                .Returns(Task.FromResult(waveitem1));
            
            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK}-BCHYDRO"))
              .Returns(Task.FromResult(waveitem2));

            _cacheServiceMock.Setup(x => x.Get($"deepsearch-{_Update_Parameter_SK}-MSDPR"))
             .Returns(Task.FromResult(waveitem3));

            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK}-ICBC",It.IsAny<object>()))
               .Returns(Task.CompletedTask);
            
            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK}-BCHYDRO", It.IsAny<object>()))
              .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.Save($"deepsearch-{_Update_Parameter_SK}-MSDPR", It.IsAny<object>()))
             .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.SearchKeys($"deepsearch-{_Update_Parameter_SK}*"))
           .Returns(Task.FromResult(new List<string>() { $"deepsearch-{_Update_Parameter_SK}-ICBC", $"deepsearch-{_Update_Parameter_SK}-BCHYDRO", $"deepsearch-{_Update_Parameter_SK}-MSDPR" }.AsEnumerable()));

           

            _cacheServiceMock.Setup(x => x.SaveRequest(It.IsAny<SearchRequest>()))
                .Returns(Task.FromResult(0));

            _cacheServiceMock.Setup(x => x.GetRequest(It.IsAny<string>()))
            .Returns(Task.FromResult(new SearchRequest { DataPartners = new List<DataPartner>() { new DataPartner { Completed = false, Name = "ICBC" }, } }));

            _cacheServiceMock.Setup(x => x.GetRequest("mykey"))
            .Returns(Task.FromResult(new SearchRequest { DataPartners = new List<DataPartner>() { new DataPartner { Completed = true, Name = "ICBC" }, } }));

            _cacheServiceMock.Setup(x => x.GetRequest("null"))
          .Returns(Task.FromResult(new SearchRequest { DataPartners = new List<DataPartner>() { new DataPartner { Completed = true, Name = "ICBC" }, } }));


            _cacheServiceMock.Setup(x => x.SearchKeys("deepsearch-null*"))
           .Returns(Task.FromResult(new List<string>() { }.AsEnumerable()));

            _cacheServiceMock.Setup(x => x.GetRequest("continuewave"))
            .Returns(Task.FromResult(new SearchRequest { DataPartners = new List<DataPartner>() { new DataPartner { Completed = true, Name = "ICBC" }, new DataPartner { Completed = true, Name = "BCHYDRO" } } }));

            _cacheServiceMock.Setup(x => x.SearchKeys("deepsearch-mykey*"))
           .Returns(Task.FromResult(new List<string>() { "first", "second", "third" }.AsEnumerable()));
            _cacheServiceMock
                .Setup(x => x.DeleteRequest(It.IsAny<string>()))
                .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.Save(It.IsAny<string>(), It.IsAny<object>())).Returns(Task.CompletedTask);

            _cacheServiceMock
              .Setup(x => x.Delete(It.IsAny<string>()))
              .Returns(Task.CompletedTask);

            _cacheServiceMock.Setup(x => x.Get("first"))
          .Returns(Task.FromResult(waveitem4));
            _cacheServiceMock.Setup(x => x.Get("second"))
          .Returns(Task.FromResult(waveitem5));
            _cacheServiceMock.Setup(x => x.Get("third"))
      .Returns(Task.FromResult(waveitem6));
            _deepSearchOptionsMock.Setup(x => x.Value).Returns(new DeepSearchOptions { MaxWaveCount = 4 });

            _searchApiNotifierMock
                .Setup(x => x.NotifyEventAsync(It.IsAny<string>(), It.IsAny<PersonSearchAdapterEvent>(), It.IsAny<string>(), It.IsAny<CancellationToken>()))
                .Returns(Task.CompletedTask);

            _dispatcherMock.Setup(x => x.StartAnotherWave(It.IsAny<string>(), It.IsAny<WaveSearchData>(), It.IsAny<Person>(), It.IsAny<int>(), It.IsAny<int>()))
                .Returns(Task.CompletedTask);

            _sut = new DeepSearchService(_cacheServiceMock.Object, _loggerMock.Object, _deepSearchOptionsMock.Object, _dispatcherMock.Object);

        }



        [Test]
        public async Task should_not_update_partner_with_is_accepted()
        {
            await _sut.UpdateDataPartner("mykey", "ICBC", "Accepted");
            await _sut.UpdateDataPartner("mykey", "ICBC", "Submitted");
            await _sut.UpdateDataPartner("mykey", "ICBC", "Finalized");
            _cacheServiceMock.Verify(x => x.SaveRequest(It.IsAny<SearchRequest>()), Times.Never());
        }

        [Test]
        public async Task should_update_partner_with_failed_completed_rejected()
        {

            await _sut.UpdateDataPartner("mykey", "ICBC", "Completed");
            await _sut.UpdateDataPartner("mykey", "ICBC", "Rejected");
            _cacheServiceMock.Verify(x => x.UpdateDataPartnerCompleteStatus(It.IsAny<string>(),It.Is<string>(m=>m=="ICBC")), Times.Exactly(2));
        }

        [Test]
        public async Task should_update_parameters_with_new_ids()
        {

            await _sut.UpdateParameters("Completed", completedStatus, _Update_Parameter_SK);
            _cacheServiceMock.Verify(x => x.SearchKeys($"deepsearch-{_Update_Parameter_SK}*"), Times
                .Once());
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK}-ICBC"), Times.Once);
            _loggerMock.VerifyLog(LogLevel.Information, $"Store all newly found  params from ICBC",Times.Once());

            _cacheServiceMock.Verify(x => x.Save(It.IsAny<string>(),It.IsAny<object>()), Times.Exactly(3));

        }

        [Test]
        public async Task should_update_parameters_with_new_with_ccn_ids()
        {

            await _sut.UpdateParameters("Completed", completedStatusCCN, _Update_Parameter_SK_CCN);
            _cacheServiceMock.Verify(x => x.SearchKeys($"deepsearch-{_Update_Parameter_SK_CCN}*"), Times
                .Once());
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-ICBC"), Times.Once);
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-MSDPR"), Times.Once);
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-WORKSAFEBC"), Times.Once);
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-BCHYDRO"), Times.Once);
            _loggerMock.VerifyLog(LogLevel.Information, $"Store all newly found  params from WORKSAFEBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"3 Identifier was returned by WORKSAFEBC", Times.Exactly(4));

            _loggerMock.VerifyLog(LogLevel.Information, $"1 returned Identifier matched the required types for ICBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"1 returned Identifier matched the required types for MSDPR", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"3 returned Identifier matched the required types for WORKSAFEBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"0 returned Identifier matched the required types for BCHYDRO", Times.Once());

            //_loggerMock.VerifyLog(LogLevel.Information, $"1 ids to be stored as new parameter for ICBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"1 ids to be stored as new parameter for MSDPR", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"2 ids to be stored as new parameter for WORKSAFEBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"0 ids to be stored as new parameter for BCHYDRO", Times.Once());
            _cacheServiceMock.Verify(x => x.Save(It.IsAny<string>(), It.IsAny<object>()), Times.Exactly(4));
            completedStatusCCN = new FakePersonSearchCompleted
            {

                SearchRequestKey = _Update_Parameter_SK,
                SearchRequestId = Guid.NewGuid(),
                ProviderProfile = new FakeProviderProfile { Name = "ICBC", SearchSpeedType = SearchSpeedType.Fast },
                MatchedPersons = new List<PersonFound> {
                    new PersonFound { Identifiers= new List<PersonalIdentifier> {


                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="5305367" },
                        new PersonalIdentifier { Type = PersonalIdentifierType.BCDriverLicense, Value="5305367" }
                    }

                }
                }
            };

            await _sut.UpdateParameters("Completed", completedStatusCCN, _Update_Parameter_SK_CCN);

            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-ICBC"), Times.Exactly(2));
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-MSDPR"), Times.Exactly(2));
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-WORKSAFEBC"), Times.Exactly(2));
            _cacheServiceMock.Verify(x => x.Get($"deepsearch-{_Update_Parameter_SK_CCN}-BCHYDRO"), Times.Exactly(2));
            _loggerMock.VerifyLog(LogLevel.Information, $"Store all newly found  params from ICBC", Times.Once() );
            _loggerMock.VerifyLog(LogLevel.Information, $"1 Identifier was returned by ICBC", Times.Exactly(4));

            _loggerMock.VerifyLog(LogLevel.Information, $"1 returned Identifier matched the required types for ICBC", Times.Exactly(2));
            _loggerMock.VerifyLog(LogLevel.Information, $"0 returned Identifier matched the required types for MSDPR", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"0 returned Identifier matched the required types for WORKSAFEBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"1 returned Identifier matched the required types for BCHYDRO", Times.Once());

            _loggerMock.VerifyLog(LogLevel.Information, $"1 ids to be stored as new parameter for ICBC", Times.Exactly(2));
            _loggerMock.VerifyLog(LogLevel.Information, $"0 ids to be stored as new parameter for MSDPR", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"0 ids to be stored as new parameter for WORKSAFEBC", Times.Once());
            _loggerMock.VerifyLog(LogLevel.Information, $"1 ids to be stored as new parameter for BCHYDRO", Times.Once());

            _cacheServiceMock.Verify(x => x.Save(It.IsAny<string>(), It.IsAny<object>()), Times.Exactly(8));
        }

        [Test]
        public async Task should_process_wave_search_initiate_new_wave()
        {
            _deepSearchOptionsMock.Setup(x => x.Value).Returns(new DeepSearchOptions
            {
                MaxWaveCount = 5
            });

            _sut = new DeepSearchService(_cacheServiceMock.Object, _loggerMock.Object, _deepSearchOptionsMock.Object, _dispatcherMock.Object);

            await _sut.IsWaveSearchReadyToFinalize("mykey");

            _cacheServiceMock.Verify(x => x.GetRequest(It.IsAny<string>()), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.SearchKeys("deepsearch-mykey*"), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.Get("first"), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.Get("second"), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.Get("third"), Times.Exactly(1));
            _dispatcherMock.Verify(x => x.StartAnotherWave(It.IsAny<string>(), It.IsAny<WaveSearchData>(), It.IsAny<Person>(), It.IsAny<int>(), It.IsAny<int>()), Times.Exactly(3));
        }

        [Test]
        public async Task should_finalize_wave_search_when_maxwave_reached()
        {
            _deepSearchOptionsMock.Setup(x => x.Value).Returns(new DeepSearchOptions
            {
                MaxWaveCount = 2
            });

            _sut = new DeepSearchService(_cacheServiceMock.Object, _loggerMock.Object, _deepSearchOptionsMock.Object, _dispatcherMock.Object);

            var finalize = await _sut.IsWaveSearchReadyToFinalize("mykey");

            _cacheServiceMock.Verify(x => x.GetRequest(It.IsAny<string>()), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.SearchKeys("deepsearch-mykey*"), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.Get("first"), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.Get("second"), Times.Exactly(1));
            _cacheServiceMock.Verify(x => x.Get("third"), Times.Exactly(1));

            Assert.AreEqual(finalize, true);
        }

        [Test]
        public async Task should_returun_true_if_wave_data_not_found()
        {

            Assert.IsTrue( await _sut.IsWaveSearchReadyToFinalize("null"));
        }
    }
}

